# kpdf.rb: ruby script (encoding: utf-8)
    # helper to convert a documents with various fonts to a pdf file
    #     based on Kramdown and Prawn
    # T. Yoshiizumi, 2015-01-28
begin
  require "kramdown"
rescue LoadError
end
begin
  require "prawn"
rescue LoadError
end

# the class to keep the font and document options information
class KpdfOpt
  include Prawn::Measurements

  def initialize(hs = nil)
    @dir = nil
    @root = 'Times-Roman'
    @header = 'Helvetica'
    @codeblock = 'Courier'
    @codespan = 'Courier'
    @table = 'Helvetica'
    @size = 12
    @dopt = {}
    @dopt2 = {}
    if hs.class == Hash
      set(hs)
    elsif ENV['RPT_WITH'] and (tex_path = ENV['TEX_PATH']) and
        test(?e, tex_path)
      tex_path = tex_path.gsub(/\\/, "/")
      @dir = tex_path.sub(/\/bin(64)?\/[0-9a-z]+\.exe$/i, "") + 
          "/share/texmf-dist/fonts/truetype/ipa"
      @root = "ipam.ttf"
      @header = @codeblock = @codespan = "ipag.ttf"
      @table = "ipam.ttf"
      tex_path = dir = nil
    end
  end
  attr_accessor :dir
  attr_accessor :size
  attr_accessor :dopt
  attr_accessor :dopt2

  def add_dir(xx)
    if @dir.class == String and xx.class == String and
        xx !~ /\/|\\/
      dir = @dir
      spc = (dir =~ /\\/ and dir !~ /\//) ? "\\" : "/"
      spc_ptn = Regexp.new("\\#{spc}$")
      dir = dir + spc  unless dir =~ spc_ptn
      xx = dir + xx
    end
    xx
  end
  private :add_dir

  def root=(val)
    @root = val
  end

  def root()
    add_dir(@root)
  end

  def header=(val)
    @header = val
  end

  def header()
    add_dir(@header)
  end

  def codeblock=(val)
    @codeblock = val
  end

  def codeblock()
    add_dir(@codeblock)
  end

  def codespan=(val)
    @codespan = val
  end

  def codespan()
    add_dir(@codespan)
  end

  def table=(val)
    @table = val
  end

  def table()
    add_dir(@table)
  end

  def read(filename)
    unless test(?f, filename)
      STDERR.puts "can't find '#{filename}'!"
      return
    end
    body = File.read(filename)
    lines = body.split(/\n/)
    lines.each do |line|
      if line =~ /^([a-z]+)\s+(.+)$/
        key = $1
        val = $2
        case key
        when 'dir' then @dir = val
        when 'root' then @root = val
        when 'header' then @header = val
        when 'codeblock' then @codeblock = val
        when 'codespan' then @codespan = val
        when 'table' then @table = val
        when 'size' then @size = eval(val)
        when 'dopt' then @dopt = eval(val)
        end
      end
    end
  end

  def font_default()
    {:dir=>nil, :root=>'Times-Roman', :header=>'Helvetica',
     :codeblock=>'Courier', :codespan=>'Courier',
     :table=>'Helvetica', :size=>12}
  end

  def dopt_default()
    {:page_size => 'A4', :page_layout => :portrait,
      :margin => "mm2pt(20)",
      :info => {
        :Creator => 'kramdown PDF converter',
        :CreationDate => "Time.now"},
      :compress => true, :optimize_objects => true}
  end

  def set_info(hs = nil)
    if hs.class != Hash
      hs = font_default()
      hs.update({:dopt=>Hash.new})
    end
    hs.each do |key, val|
      case key
      when :dir then @dir = val
      when :root then @root = val
      when :header then @header = val
      when :codeblock then @codeblock = val
      when :codespan then @codespan = val
      when :table then @table = val
      when :size then @size = val
      when :dopt then @dopt = val
      end
    end
  end

  def get_info()
    {:dir=>@dir, :root=>@root, :header=>@header, :codeblock=>@codeblock,
     :codespan=>@codespan, :table=>@table, :size=>@size, :dopt=>@dopt}
  end
end


# Kpdf class definition
if defined?(Kramdown)
  module Kramdown
    @@kpdf = KpdfOpt.new

    def self.kpdf()
      @@kpdf
    end

    module Converter
      class Kpdf < Pdf
        def root_options(root, opts)
          super.update({:font => ::Kramdown::kpdf.root,
            :size => ::Kramdown::kpdf.size})
        end

        def header_options(el, opts)
          super.update({:font => ::Kramdown::kpdf.header})
        end

        def codeblock_options(el, opts)
          super.update({:font => ::Kramdown::kpdf.codeblock})
        end

        def codespan_options(el, opts)
          super.update({:font => ::Kramdown::kpdf.codespan})
        end

        def render_table(el, opts)
          @pdf.font ::Kramdown::kpdf.table
          res = super
          @pdf.font 'Helvetica'
          return res
        end

        def document_options(root)
          hs = super
          dopt = ::Kramdown::kpdf.dopt
          if dopt.class == Hash and dopt.size > 0
            dopt.each do |key, val|
              if val.class == Hash
                dopt[key].each do |key2, val2|
                  if val2.class == String and
                      val2 =~ /^\s*(mm2pt\(.+\)|Time\.|Time\:\:)/
                    dopt[key][key2] = eval(val2)
                  end
                end
              elsif val.class == String and
                  val =~ /^\s*(mm2pt\(.+\)|Time\.|Time\:\:)/
                dopt[key] = eval(val)
              end
            end
            dopt.each do |key, val|
              if val.class == Hash
                if hs[key].class == Hash
                  hs[key].update(dopt[key])
                else
                  hs[key] = dopt[key]
                end
              else
                hs[key] = dopt[key]
              end
            end
            hs.delete_if {|key, val|  val == nil}
            hs.each do |key, val|
              if val.class == Hash
                hs[key].delete_if {|key2, val2|  val2 == nil}
              end
            end
          end
          ::Kramdown::kpdf.dopt2 = hs
          return hs
        end
      end
    end
  end
end
